/*
 * Copyright ©2023 Chris Thachuk.  All rights reserved.  Permission is
 * hereby granted to students registered for University of Washington
 * CSE 333 for use solely during Fall Quarter 2023 for purposes of
 * the course.  No other use, copying, distribution, or modification
 * is permitted without prior written consent. Copyrights for
 * third-party components of this work must be honored.  Instructors
 * interested in reusing these course materials should contact the
 * author.
 */

#include "dnautil.h"

#include <random>
#include <immintrin.h>

ShortOligo RandomShortOligo(std::mt19937 &rng) {
  std::uniform_int_distribution<std::mt19937::result_type> dist(0,3);
  ShortOligo o;
  for(int i = 0; i < SHORTOLIGOLEN; ++i) {
      Base b = static_cast<Base>(dist(rng));
      o.bases[i] = b;
  }
  return o;
}

int MinPairwiseHammingDistance(const std::vector<ShortOligo> &pool) {
  const int kPoolSize = pool.size();
  int min_distance = SHORTOLIGOLEN;

  for(int i = 0; i < kPoolSize - 1; ++i) {
     auto o1 = pool[i];
     for(int j = i + 1; j < kPoolSize; ++j) {
         auto o2 = pool[j];
         #ifndef SENDIT
         int distance = HammingDistance(o1, o2);
         #else
         int distance = HammingDistanceAVX2(o1, o2);
         #endif
         min_distance = std::min(distance, min_distance);
     }
  }

  return min_distance;
}

int HammingDistance(const ShortOligo &o1, const ShortOligo &o2) {
  int distance = 0;
  for(int i = 0; i < SHORTOLIGOLEN; ++i) {
    if( o1.bases[i] != o2.bases[i] ) {
      distance += 1;
    }
  }
  return distance;
}

#ifdef SENDIT
int HammingDistanceAVX2(ShortOligo &o1, ShortOligo &o2) {
  const __m256i l1 = _mm256_loadu_si256((const __m256i*)o1.bases);
  const __m256i l2 = _mm256_loadu_si256((const __m256i*)o2.bases);

  auto eq_lo = _mm256_cmpeq_epi8(l1, l2);
  int mask_lo = _mm256_movemask_epi8(eq_lo);
  int similarity_lo =  _mm_popcnt_u32(mask_lo);

  return SHORTOLIGOLEN - similarity_lo;
}
#endif

BaseFreq BaseFrequency(const ShortOligo &o1)  {
  BaseFreq freq;
  freq.count[0] = freq.count[1] = freq.count[2] = freq.count[3] = 0;

  for(int i = 0; i < SHORTOLIGOLEN; ++i) {
    freq.count[o1.bases[i]] += 1;
  }
  return freq;
}

bool HammingDistanceOk(ShortOligo &o1, ShortOligo &o2, int threshold) {
  int distance = 0;
  for(int i = 0; i < SHORTOLIGOLEN; ++i) {
    if( o1.bases[i] != o2.bases[i] ) {
      distance += 1;
      if (distance > threshold) {
        return false;
      }
    }
  }
  return true;
}
