/*
 * Copyright ©2023 Chris Thachuk.  All rights reserved.  Permission is
 * hereby granted to students registered for University of Washington
 * CSE 333 for use solely during Fall Quarter 2023 for purposes of
 * the course.  No other use, copying, distribution, or modification
 * is permitted without prior written consent. Copyrights for
 * third-party components of this work must be honored.  Instructors
 * interested in reusing these course materials should contact the
 * author.
 */

#ifndef DNAPOOL_H_
#define DNAPOOL_H_

#include <random>
#include <immintrin.h>

#define SHORTOLIGOLEN 32

enum Base: unsigned char
{
    A = 0,
    C = 1,
    G = 2,
    T = 3,
};

struct ShortOligo {
    Base bases[SHORTOLIGOLEN];
};

struct BaseFreq {
    int count[4];
};

// Returns a random ShortOligo.
ShortOligo RandomShortOligo(std::mt19937 &rng);

// Returns the Hamming Distance between o1 and o2.
// Specifically, it counts the number of positions
// in the oligo where a and b have different bases.
int HammingDistance(const ShortOligo &o1, const ShortOligo &o2);

bool HammingDistanceOk(ShortOligo &o1, ShortOligo &o2, int threshold);

int MinPairwiseHammingDistance(const std::vector<ShortOligo> &pool);

#ifdef SENDIT
int HammingDistanceAVX2(ShortOligo &o1, ShortOligo &o2);
#endif

#endif // DNAPOOL_H_
