#include <random>
#include <benchmark/benchmark.h>

#include "dnautil.h"

static void BM_RandomShortOligo(benchmark::State& state) {
  std::random_device dev;
  std::mt19937 rng(dev());

  for (auto _ : state) {
    RandomShortOligo(rng);
  }
}
// Register the function as a benchmark
BENCHMARK(BM_RandomShortOligo);

static void BM_HammingDistance(benchmark::State& state) {
  std::random_device dev;
  std::mt19937 rng(dev());
  auto o1 = RandomShortOligo(rng);
  auto o2 = RandomShortOligo(rng);

  for (auto _ : state) {
    HammingDistance(o1, o2);
  }
}
// Register the function as a benchmark
BENCHMARK(BM_HammingDistance);

#ifdef SENDIT
static void BM_HammingDistanceAVX2(benchmark::State& state) {
  std::random_device dev;
  std::mt19937 rng(dev());
  auto o1 = RandomShortOligo(rng);
  auto o2 = RandomShortOligo(rng);

  for (auto _ : state) {
    HammingDistanceAVX2(o1, o2);
  }
}
// Register the function as a benchmark
BENCHMARK(BM_HammingDistanceAVX2);
#endif

static void BM_HammingDistanceOk(benchmark::State& state) {
  std::random_device dev;
  std::mt19937 rng(dev());
  auto o1 = RandomShortOligo(rng);
  auto o2 = RandomShortOligo(rng);

  for (auto _ : state) {
    HammingDistanceOk(o1, o2, 16);
  }
}
// Register the function as a benchmark
BENCHMARK(BM_HammingDistanceOk);

BENCHMARK_MAIN();
