/* Copyright ©2023 Chris Thachuk.  All rights reserved.  Permission is
 * hereby granted to students registered for University of Washington
 * CSE 333 for use solely during Fall Quarter 2023 for purposes of
 * the course.  No other use, copying, distribution, or modification
 * is permitted without prior written consent. Copyrights for
 * third-party components of this work must be honored.  Instructors
 * interested in reusing these course materials should contact the
 * author.
 */

#include <chrono>
#include <iostream>
#include <random>
#include <vector>
#include <cstdlib>

#include "dnautil.h"

#define TOOSIMILAR 16

using std::cerr;
using std::endl;
using std::cout;

// Prints usage information about this program and exits with EXIT_FAILURE.
void Usage(char *progname);

int main(int argc, char **argv) {
  if (argc != 2) {
    Usage(argv[0]);
  }

  int poolsize = atoi(argv[1]);

  std::vector<ShortOligo> pool;
  pool.reserve(poolsize);

  std::random_device dev;
  std::mt19937 rng(dev());

  for(int i = 0; i < poolsize; ++i) {
      pool.push_back(RandomShortOligo(rng));
  }

  auto start = std::chrono::high_resolution_clock::now();
  int min_distance = MinPairwiseHammingDistance(pool);
  auto stop = std::chrono::high_resolution_clock::now();

  auto duration = std::chrono::duration_cast<std::chrono::milliseconds>(stop - start);
  cout << "Minimum distance : " << min_distance << endl;
  cout << "Time to calc (ms): " << duration.count() << endl;

  return EXIT_SUCCESS;
}

void Usage(char *progname) {
  cerr << "usage: " << progname << " num_strands" << endl;
  exit(EXIT_FAILURE);
}

